// (C) 2003-2015 Doctor Web. All rights reserved.

#ifndef DRWEB_API_IPC_EXAMPLE_H
#define DRWEB_API_IPC_EXAMPLE_H

#include <functional>
#include <memory>
#include <stdexcept>
#include <vector>

#include <boost/asio.hpp>
#include <boost/system/system_error.hpp>

#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/io/zero_copy_stream_impl_lite.h>
#include <google/protobuf/message.h>

class ipc_t : public std::enable_shared_from_this<ipc_t>
{
public:
    explicit ipc_t(boost::asio::io_service& io);
    virtual ~ipc_t();

protected:
    void async_connect(const std::string& path);
    void async_send_message(const google::protobuf::Message& msg);
    void close_ipc();

    virtual void message_received(google::protobuf::io::ZeroCopyInputStream* is) = 0;
    virtual void ipc_error_occurred(const boost::system::error_code& ec) = 0;
    virtual std::shared_ptr<ipc_t> this_as_ipc() = 0;

private:

    void handle_async_connect(const boost::system::error_code& ec);

    void read_message_size();
    void handle_read_message_size(const boost::system::error_code& ec, size_t size);
    void read_message(size_t message_size);
    void handle_read_message(const boost::system::error_code& ec, size_t size);

    void write_next();
    void handle_write(const boost::system::error_code& ec, size_t size);

    bool is_open() const;

    boost::asio::io_service& io_;
    std::shared_ptr<boost::asio::local::stream_protocol::socket> socket_;
    std::vector<uint8_t> receive_buffer_;
    std::vector<uint8_t> send_buffer_;
    std::vector<uint8_t> sending_buffer_;
};

#endif // DRWEB_API_IPC_EXAMPLE_H
