// (C) 2003-2015 Doctor Web. All rights reserved.

#ifndef DRWEB_NETCHECK_API_H
#define DRWEB_NETCHECK_API_H

#include <unistd.h>

#include <cstdint>
#include <iostream>
#include <map>

#include "drweb_api_error.pb.h"
#include "drweb_ipc.h"
#include "drweb_netcheck_api.pb.h"

class netcheck_api_t : public ipc_t
{
public:
    // NetCheck job status report interface
    struct delegate_t {
        virtual void on_scan_report(uint32_t job, const nc::ScanReport& report) = 0;
        virtual void on_cured_data_size(uint32_t job, off_t size) = 0;
        virtual void on_cured_data(uint32_t job, const void* data, size_t size) = 0;
        virtual void on_alarm(uint32_t job, dw::ErrorCode ec) = 0;
        virtual void on_window_replenished() = 0;
    };

public:
    static std::shared_ptr<netcheck_api_t> create(boost::asio::io_service& io);

    void start(const std::string& path);
    void stop();

    uint32_t init_job(delegate_t* delegate);
    void send_data(uint32_t job, const void* data, size_t size);
    void send_scan_request(uint32_t job, const nc::ScanOptions& options);
    void send_get_data_request(uint32_t job);
    void send_reset(uint32_t job);
    int32_t window_size() const { return window_size_; }

private:
    // ipc_t
    void message_received(google::protobuf::io::ZeroCopyInputStream* is) override final;
    void ipc_error_occurred(const boost::system::error_code& ec) override final;
    virtual std::shared_ptr<ipc_t> this_as_ipc() override final;

private:
    explicit netcheck_api_t(boost::asio::io_service& io);

    void send_hello();

    void handle_scan_response(uint32_t job, const nc::ScanReport& report);
    void handle_get_data_response(uint32_t job, off_t size);
    void handle_data_chunk(uint32_t job, const std::string& data);
    void handle_alarm(uint32_t job, dw::ErrorCode ec);

    uint32_t serial_;
    std::map<uint32_t, delegate_t*> jobs_;
    int32_t window_size_;
};

#endif // DRWEB_NETCHECK_API_H
